"""
指定 filelist，将其中的所有文件复制到指定 path
"""
import os
import threading
import multiprocessing
from hhcp.utils.chunks import get_chunks


def cp_list(filelist, destination, prefix, overwrite):
    class myThread(threading.Thread):
        def __init__(self, threadID, filelist, destination, overwrite):
            threading.Thread.__init__(self)
            self.threadID = threadID
            self.filelist = filelist
            self.destination = destination
            self.overwrite = overwrite

        def run(self):
            print("开启线程： " + self.name)
            # 获取锁，用于线程同步
            # threadLock.acquire()

            for i, f in enumerate(self.filelist):
                if self.overwrite:
                    command = "cp -r {} {}".format(f, self.destination)
                else:
                    command = "cp -r -n {} {}".format(f, self.destination)
                os.system(command)
                print("Thread ID {}, {}/{}, {}".format(self.threadID, i, len(self.filelist), command))

    thread_num = multiprocessing.cpu_count()

    files = []
    for l in open(filelist):
        files.append(os.path.join(prefix, l.strip("\n")))

    filechunks = get_chunks(files, thread_num)
    thread_num = len(filechunks)

    threadLock = threading.Lock()
    threads = []

    for i in range(thread_num):
        thread = myThread(i, filechunks[i], destination)
        threads.append(thread)
        thread.start()

    # 等待所有线程完成
    for t in threads:
        t.join()
    print("cp 结束，退出主线程")



"""
指定路径，将其中的所有文件复制到指定 path
"""

import os
import threading
import multiprocessing
from hhcp.utils.chunks import get_chunks

def cp_dir(directory, destination, force):
    class myThread(threading.Thread):
        def __init__(self, threadID, filelist, overwrite):
            threading.Thread.__init__(self)
            self.threadID = threadID
            self.filelist = filelist
            self.overwrite = overwrite

        def run(self):
            print("开启线程： " + self.name)
            # 获取锁，用于线程同步
            # threadLock.acquire()
            i = 1
            for f in self.filelist:
                if len(f) == 1:
                    print("thread ID {}, {}, try create path [{}]".format(self.threadID, i, f[0]))
                    if not os.path.exists(f[0]):
                        try:
                            os.makedirs(f[0])
                            print("thread ID {}, {}, path [{}] ........ created!".format(self.threadID, i, f[0]))
                        except FileExistsError:
                            print("thread ID {}, {}, path [{}] already exists".format(self.threadID, i, f[0]))
                    else:
                        print("thread ID {}, {}, path [{}] already exists".format(self.threadID, i, f[0]))
                else:
                    print("thread ID {}, {}, try copy [{}] to [{}]".format(self.threadID, i, f[0], f[1]))
                    if not os.path.exists(f[1]):
                        print("thread ID {}, {}, folder [{}] does not exist".format(self.threadID, i, f[1]))
                        print("thread ID {}, {}, try create path [{}]".format(self.threadID, i, f[1]))
                        try:
                            os.makedirs(f[1])
                            print("thread ID {}, {}, path [{}] ........ created!".format(self.threadID, i, f[1]))
                        except FileExistsError:
                            print("thread ID {}, {}, path [{}] already exists".format(self.threadID, i, f[1]))
                    if self.overwrite:
                        command = "cp {} {}".format(f[0], f[1])
                    else:
                        command = "cp -n {} {}".format(f[0], f[1])
                    os.system(command)
                    print("thread ID {}, copy [{}] to [{}] ........ successed!".format(self.threadID, i, command))
                i += 1

    thread_num = multiprocessing.cpu_count()

    absp = os.path.abspath(directory)
    absp_prefix = os.path.split(absp)[0]
    absd = os.path.abspath(destination)

    filelist = []
    subs = os.walk(directory)
    for sub in subs:
        pre = sub[0]
        absd_suffix = os.path.relpath(os.path.abspath(pre), absp_prefix)
        files = sub[2]
        if len(files) != 0:
            """
            有文件的时候，filelist 里存为
            [原始文件绝对路径, 要 cp 到的文件夹的绝对路径]
            """
            for file in files:
                filelist.append([os.path.join(os.path.abspath(pre), file), os.path.join(absd, absd_suffix)])
        else:
            """
            没有文件的时候，filelist 里存为
            [要 cp 成为的文件夹的绝对路径]
            """
            filelist.append([os.path.join(absd, absd_suffix)])

    filechunks = get_chunks(filelist, thread_num)
    thread_num = len(filechunks)

    threadLock = threading.Lock()
    threads = []

    for i in range(thread_num):
        thread = myThread(i + 1, filechunks[i], force)
        threads.append(thread)
        thread.start()

    # 等待所有线程完成
    for t in threads:
        t.join()
    print("cp 结束，退出主线程")



import argparse
import os
from argparse import RawTextHelpFormatter
import sys
from hhcp.cp_dir_multi import cp_dir
from hhcp.cp_list_multi import cp_list


def parse_args():
    parser = argparse.ArgumentParser(
        description="""
        Provide (a file list)|(a dir) and a destination, cp (all the files in the list)|(the dir) to the destination with multi-thread.
        This will NOT overwrite existing files on default.\n
        """,
        formatter_class=RawTextHelpFormatter
    )
    parser.add_argument(
        "-p",
        help="The path of the directory to be copied.",
        default=None,
        type=str,
    )
    parser.add_argument(
        "-f",
        help="The location of the file list.",
        default=None,
        type=str,
    )
    parser.add_argument(
        "-d",
        required=True,
        help="The path of the destination",
        default=None,
        type=str,
    )
    parser.add_argument(
        "--force",
        help="add this to allow force overwrite",
        action="store_true",
        default=False,
    )
    parser.add_argument(
        "--pre",
        help="The path prefix of the files in the list",
        default=None,
        type=str,
    )
    if len(sys.argv) == 1:
        parser.print_help()
    return parser.parse_args()


def load_config(args):
    """
    Given the arguemnts, load and initialize the configs.
    Args:
        args (argument): arguments includes `p`, `d`, `n`, `force`
    """
    if args.p is not None and args.f is not None:
        print("Only one of -p and -f could be provided but get two.")
        sys.exit(0)
    elif args.p is None and args.f is None:
        print("At least one of -p and -f should be provided.")
        sys.exit(0)
    else:
        # 判断 p 是否为合法路径
        if args.p is not None:
            if not os.path.exists(args.p):
                print("The dir to be copied does not exist.")
                sys.exit(0)
            elif not os.path.isdir(args.p):
                print("-p should be a dir.")
                sys.exit(0)
        # 判断 f 是否为合法路径
        elif args.f is not None:
            if not os.path.exists(args.f):
                print("File not found.")
                sys.exit(0)
    # 判断 d 是否为合法路径
    if args.d is not None:
        if not os.path.isdir(args.d):
            print("-d should be a dir.")
            sys.exit(0)
        if not os.path.exists(args.d):
            print("The destination dir does not exist.")
            sys.exit(0)
    if args.pre is None:
        args.pre = ''
    return args.p, args.f, args.d, args.pre, args.force


def main():
    directory, filelist, destination, prefix, force = load_config(parse_args())
    if filelist is not None:
        cp_list(filelist, destination, prefix, force)
    elif directory is not None:
        cp_dir(directory, destination, force)
    else:
        print("At least one of -p and -f should be provided.")


if __name__ == "__main__":
    main()
